@@ -67,7 +67,11 @@
      (doseq [child# ~children]
        (try
          (child# ~event)
-         (catch Throwable e#
+         (catch Exception e#
+           (if-let [ex-stream# *exception-stream*]
+             (ex-stream# (exception->event e# ~event))
+             (warn e# (str child# " threw"))))
+         (catch AssertionError e#
            (if-let [ex-stream# *exception-stream*]
              (ex-stream# (exception->event e# ~event))
              (warn e# (str child# " threw"))))))
@@ -182,6 +186,13 @@
     (doseq [e (f event)]
       (call-rescue e children))))
 
+(defn sflatten
+  "Streaming flatten. Calls children with each event in events. Events should be a sequence."
+  [& children]
+  (fn stream [events]
+    (doseq [e events]
+      (call-rescue e children))))
+
 (defn sreduce
   "Streaming reduce. Two forms:
 
@@ -298,25 +309,30 @@
 
   Events without times accrue in the current window."
   [n & children]
-  (let [cutoff (ref 0)
-        buffer (ref [])]
+  (let [state (atom {:cutoff 0
+                     :buffer []
+                     :send? true})]
     (fn stream [event]
-      (let [events (dosync
-                     ; Compute minimum allowed time
-                     (let [cutoff (alter cutoff max (- (get event :time 0) n))]
-                       (when (or (nil? (:time event))
-                                 (< cutoff (:time event)))
-                         ; This event belongs in the buffer, and our cutoff may
-                         ; have changed.
-                         (alter buffer conj event)
-                         (alter buffer
-                                (fn alter [events]
-                                  (vec (filter
-                                         (fn [e] (or (nil? (:time e))
-                                                     (< cutoff (:time e))))
-                                         events)))))))]
-        (when events
-          (call-rescue events children))))))
+      (let [result (swap!
+                    state
+                    (fn [{:keys [cutoff buffer]}]
+                      ; Compute minimum allowed time
+                      (let [cutoff (max cutoff (- (get event :time 0) n))
+                            send? (or (nil? (:time event))
+                                      (< cutoff (:time event)))
+                            buffer (if send?
+                                        ; This event belongs in the buffer,
+                                        ; and our cutoff may have changed.
+                                     (vec (filter
+                                           (fn [e] (or (nil? (:time e))
+                                                       (< cutoff (:time e))))
+                                           (conj buffer event)))
+                                     buffer)]
+                        {:cutoff cutoff
+                         :buffer buffer
+                         :send? send?})))]
+        (when (:send? result)
+          (call-rescue (:buffer result) children))))))
 
 (defn- fixed-time-window-fn
   "A fixed window over the event stream in time. Emits vectors of events, such
@@ -324,7 +340,6 @@
   *not* overlap; each event appears at most once in the output stream. Once an
   event is emitted, all events *older or equal* to that emitted event are
   silently dropped.
-
   Events without times accrue in the current window."
   [n start-time-fn & children]
   ; This is not a particularly inspired or clear implementation. :-(
@@ -332,44 +347,43 @@
   (when (zero? n)
     (throw (IllegalArgumentException. "Can't have a zero-width time window.")))
 
-  (let [start-time (ref nil)
-        buffer     (ref [])]
+  (let [state (atom {:start-time nil
+                     :buffer []
+                     :windows nil})]
     (fn stream [event]
-      (let [windows (dosync
-                      (cond
-                        ; No time
-                        (nil? (:time event))
-                        (do
-                          (alter buffer conj event)
-                          nil)
+      (let [s (swap! state
+                     (fn [{:keys [start-time buffer] :as state}]
+                       (cond
+                         ; No time
+                         (nil? (:time event))
+                         (-> (update state :buffer conj event)
+                             (assoc :windows nil))
 
-                        ; No start time
-                        (nil? @start-time)
-                        (do
-                          (ref-set start-time (start-time-fn n event))
-                          (ref-set buffer [event])
-                          nil)
+                         ; No start time
+                         (nil? start-time)
+                         (assoc state :start-time (start-time-fn n event)
+                                      :buffer [event]
+                                      :windows nil)
 
-                        ; Too old
-                        (< (:time event) @start-time)
-                        nil
+                         ; Too old
+                         (< (:time event) start-time)
+                         (assoc state :windows nil)
 
-                        ; Within window
-                        (< (:time event) (+ @start-time n))
-                        (do
-                          (alter buffer conj event)
-                          nil)
+                         ; Within window
+                         (< (:time event) (+ start-time n))
+                         (-> (update state :buffer conj event)
+                             (assoc :windows nil))
 
-                        ; Above window
-                        true
-                        (let [delta (- (:time event) @start-time)
-                              dstart (- delta (mod delta n))
-                              empties (dec (/ dstart n))
-                              windows (conj (repeat empties []) @buffer)]
-                          (alter start-time + dstart)
-                          (ref-set buffer [event])
-                          windows)))]
-        (when windows
+                         ; Above window
+                         true
+                         (let [delta (- (:time event) start-time)
+                               dstart (- delta (mod delta n))
+                               empties (dec (/ dstart n))
+                               windows (conj (repeat empties []) buffer)]
+                           (-> (update state :start-time + dstart)
+                               (assoc :buffer [event]
+                                      :windows windows))))))]
+        (when-let [windows (:windows s)]
           (doseq [w windows]
             (call-rescue w children)))))))
 
@@ -1059,9 +1073,10 @@
            bottom-stream))))
 
 (defn throttle
-  "Passes on at most n events every dt seconds. If more than n events arrive in
-  a dt-second fixed window, drops remaining events. Imposes no additional
-  latency; events are either passed on immediately or dropped."
+  "Passes on at most n events, or vectors of events, every dt seconds. If more
+  than n events (or vectors) arrive in a dt-second fixed window, drops
+  remaining events. Imposes no additional latency; events are either passed on
+  immediately or dropped."
   [n dt & children]
   (part-time-simple
     dt
@@ -1280,9 +1295,18 @@
 (def tagged "Alias for tagged-all" tagged-all)
 
 (defn expired
-  "Passes on events with :state \"expired\"."
+  "Passes on expired events."
   [& children]
-  (apply match :state "expired" children))
+  (fn stream [event]
+    (when (expired? event)
+      (call-rescue event children))))
+
+(defn not-expired
+  "Passes on not expired events."
+  [& children]
+  (fn stream [event]
+    (when (not (expired? event))
+      (call-rescue event children))))
 
 (defn with
   "Constructs a copy of each incoming event with new values for the given keys,
@@ -1508,7 +1532,7 @@
    (by-builder [host :host] (forward (get relay-by-host host)))
    "
   [[sym fields] & forms]
-  `(let [new-fork# (fn [~sym] [(do ~@forms)])]
+  `(let [new-fork# (fn [~sym] [~@forms])]
      (by-fn ~fields new-fork#)))
 
 (defn by-fn [fields new-fork]
@@ -1709,7 +1733,7 @@
         [true-kids else-kids] (where-partition-clauses children)]
     `(let [true-kids# ~true-kids
            else-kids# ~else-kids]
-       (fn stream [event#]
+       (fn stream# [event#]
          (let [value# (let [~'event event#] ~p)]
            (if value#
              (call-rescue event# true-kids#)
